% trimOutlier() - rejects 1.channels that are below or above the specified
%                 SD, and 2.datapoints that are above the specified
%                 threshold. Point spread width [ms] determines an range
%                 extention for rejection.  
%
% Usage:
%   >> EEG = trimOutlier(EEG, channelSdLowerBound, channelSdUpperBound, amplitudeThreshold);

% History:
% 04/01/2014 ver 1.2 by Makoto. Check inputs.
% 03/26/2014 ver 1.1 by Makoto. Debug and simplify datapoint rejection
% 03/07/2014 ver 1.0 by Makoto. Former firstpassOutlierTrimmer redesigned and renamed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 09/06/2013 ver 1.5 by Makoto. No datapoint rejection reflects channel rejection.
% 08/05/2013 ver 1.4 by Makoto. Supported 3-D data (except for datapoint rejection)
% 06/27/2013 ver 1.3 by Makoto. Error message for inputting 3-D data.
% 06/13/2013 ver 1.2 by Makoto. Scalp topos added (when valid channel data exist)
% 05/22/2013 ver 1.1 by Makoto. Minor brush up.
% 05/16/2013 ver 1.0 by Makoto. Created.

% Author: Makoto Miyakoshi, SCCN,INC,UCSD 2013
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function EEG = trimOutlier(EEG, channelSdLowerBound, channelSdUpperBound, amplitudeThreshold, pointSpreadWidth)

if ~(nargin==5)
    error('trimOutlier() requires 5 input arguments.')
end

% remove bad channels with SD
stdAllPnts = std(EEG.data(:,:),0,2);
badChanIdx = find((stdAllPnts < channelSdLowerBound) | (stdAllPnts > channelSdUpperBound));
EEG = pop_select(EEG, 'nochannel', badChanIdx);

% return if 3-D
if length(size(EEG.data))==3
    disp('Epoched data detected: datapoint rejection will not be performed.')
    return
end

%% remove bad datapoints

% calculate half window size
windowSize = pointSpreadWidth; % millisecond
halfWinSize = floor(round(windowSize/(1000/EEG.srate))/2);

% compute bad datapoints
absMinMaxAllChan = max([abs(min(EEG.data)); abs(max(EEG.data))],[],1);
badPntsCenter = find(absMinMaxAllChan > amplitudeThreshold);

% expand bad points to +/- half window size 
badPntsStart = (badPntsCenter-halfWinSize)'; badPntsStart(badPntsStart<0)=0;
badPntsEnd   = (badPntsCenter+halfWinSize+1)'; badPntsEnd(badPntsEnd>EEG.pnts)=EEG.pnts;
rejectPntsStartEnd = [badPntsStart badPntsEnd];

% compute overlaps
badPnts = [];
for n = 1:length(badPntsStart)
    badPnts = unique([badPnts badPntsStart(n):badPntsEnd(n)]);
end
diffbadPnts = diff(badPnts);
gapIdx      = find(diffbadPnts~=1);
startIdx    = [badPnts(1) badPnts(gapIdx+1)]';
endIdx      = [badPnts(gapIdx) badPnts(end)]';

% reject them
EEG = pop_select(EEG, 'nopoint', [startIdx endIdx]);